#include <stdio.h>
#include <stdlib.h>

struct int_node {
  int val;
  struct int_node * prev; /* NULL => no previous elements */
  struct int_node * next; /* NULL => empty */
};

typedef struct int_node * node;

node make_node(node prev_ls, int i, node next_ls) {
  node ls2;
  
  ls2 = (node)malloc(sizeof(struct int_node));
  
  ls2->val = i;
  ls2->prev = prev_ls;
  ls2->next = next_ls;

  if (next_ls != NULL)
    next_ls->prev = ls2;
  if (prev_ls != NULL)
    prev_ls->next = ls2;
  
  return ls2;
}

/* We can treat a `node' as starting a list going forward: */
void print_list(struct int_node * ls) {
  for (; ls != NULL; ls = ls->next) {
    printf("%d\n", ls->val);
  }
}

/* We can treat a `node' as starting a list going backward: */
void print_list_backwards(struct int_node * ls) {
  for (; ls != NULL; ls = ls->prev) {
    printf("%d\n", ls->val);
  }
}

/* ---------------------------------------- */

struct container {
  int count;
  node hd;
  node tl;
};

typedef struct container * list_container;

list_container make_list_container() {
  list_container lc;

  lc = (list_container)malloc(sizeof(struct container));
  lc->count = 0;
  lc->hd = NULL;
  lc->tl = NULL;

  return lc;
}

void add_to_front(list_container lc, int i) {
  lc->hd = make_node(NULL, i, lc->hd);
  if (lc->tl == NULL)
    lc->tl = lc->hd;
  lc->count++;
}

void add_to_back(list_container lc, int i) {
  lc->tl = make_node(lc->tl, i, NULL);
  if (lc->hd == NULL)
    lc->hd = lc->tl;
  lc->count++;
}

int remove_from_front(list_container lc) {
  int v;

  v = lc->hd->val;
  lc->hd = lc->hd->next;
  if (lc->hd)
    lc->hd->prev = NULL;
  else
    lc->tl = NULL;    
  --lc->count;

  return v;
}

int remove_from_back(list_container lc) {
  int v;

  v = lc->tl->val;

  lc->tl = lc->tl->prev;
  if (lc->tl)
    lc->tl->next = NULL;
  else
    lc->hd = NULL;
  --lc->count;

  return v;
}

int get_count(list_container lc) {
  return lc->count;
}

int get_value(list_container lc, int pos) {
  node nd;
  for (nd = lc->hd; pos-- > 0; nd = nd->next) {
  }
  return nd->val;
}

/* ---------------------------------------- */

/* Tests */

void failed() {
  printf("test failed\n");
  exit(1);
}

void check_ptr(void *a, void *b) {
  if (a != b) failed();
}

void check_int(int a, int b) {
  if (a != b) failed();
}

void test() {
  list_container lc;
  int i, j;

  lc = make_list_container();
  check_int(get_count(lc), 0);
  add_to_front(lc, 5);
  check_int(get_count(lc), 1);
  check_int(get_value(lc, 0), 5);
  add_to_back(lc, 6);
  check_int(get_count(lc), 2);
  check_int(get_value(lc, 0), 5);
  check_int(get_value(lc, 1), 6);
  add_to_back(lc, 7);
  add_to_front(lc, 4);
  check_int(get_count(lc), 4);
  check_int(get_value(lc, 0), 4);
  check_int(get_value(lc, 1), 5);
  check_int(get_value(lc, 2), 6);
  check_int(get_value(lc, 3), 7);

  /* Check behavior as a queue: */
  lc = make_list_container();
  for (i = 0; i < 100; i++) {
    for (j = 0; j < i; j++)
      add_to_front(lc, j);
    check_int(get_count(lc), i);
    for (j = 0; j < i; j++)
      check_int(remove_from_back(lc), j);

    check_int(get_count(lc), 0);

    for (j = 0; j < i; j++)
      add_to_back(lc, j);
    check_int(get_count(lc), i);
    for (j = 0; j < i; j++)
      check_int(remove_from_front(lc), j);
  }

  /* Check behavior as a stack: */
  lc = make_list_container();
  for (i = 0; i < 100; i++) {
    for (j = 0; j < i; j++)
      add_to_front(lc, j);
    check_int(get_count(lc), i);
    for (j = i; j-- > 0; )
      check_int(remove_from_front(lc), j);

    check_int(get_count(lc), 0);

    for (j = 0; j < i; j++)
      add_to_back(lc, j);
    check_int(get_count(lc), i);
    for (j = i; j-- > 0; )
      check_int(remove_from_back(lc), j);
  }
}

/* ---------------------------------------- */

int main() {
  list_container lc;
  char buffer[256];
  int i, count, back;

  test();

  lc = make_list_container();
  
  for (back = 1; fgets(buffer, 256, stdin); back = !back) {
    if (back)
      add_to_back(lc, atoi(buffer));
    else
      add_to_front(lc, atoi(buffer));
  }

  count = get_count(lc);
  for (i = 0; i < count; i++) {
    back = !back;
    if (back)
      printf("%d\n", remove_from_back(lc));
    else
      printf("%d\n", remove_from_front(lc));
  }
  
  return 0;
}
